"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const api_1 = require("../api/api");
const constants_1 = require("../utils/constants");
const chalk = require("chalk");
const figures = require("figures");
const _ = require("lodash");
const file_1 = require("../utils/file");
const { terminal } = require('terminal-kit');
const debug = require('debug')('graphcool');
exports.default = (props, env) => __awaiter(this, void 0, void 0, function* () {
    const { resolver, out } = env;
    if (props.sourceProjectId) {
        out.startSpinner(constants_1.deletingProjectMessage(props.sourceProjectId));
        try {
            yield api_1.deleteProject([props.sourceProjectId], resolver);
            out.stopSpinner();
            out.write(constants_1.deletedProjectMessage([props.sourceProjectId]));
        }
        catch (e) {
            out.stopSpinner();
            if (e.errors) {
                const errors = api_1.parseErrors(e);
                const output = api_1.generateErrorOutput(errors);
                out.writeError(`${output}`);
            }
            else {
                throw e;
            }
        }
    }
    else {
        const projects = yield api_1.fetchProjects(resolver);
        terminal.saveCursor();
        terminal.grabInput();
        terminal.hideCursor();
        terminal(`\n`);
        // initially select current project
        const projectId = getProjectId(env);
        let currentIndex = projectId ? projects.map(p => p.projectId).indexOf(projectId) : 0;
        const selectedIndices = [];
        render(projects, currentIndex, selectedIndices);
        yield new Promise(resolve => {
            terminal.on('key', (name) => __awaiter(this, void 0, void 0, function* () {
                currentIndex = yield handleKeyEvent(name, currentIndex, selectedIndices, projects, env, resolve);
            }));
        });
    }
});
function getProjectId(env) {
    const projectFiles = env.resolver.projectFiles('.');
    if (projectFiles.length !== 1) {
        return;
    }
    const projectFile = projectFiles[0];
    if (projectFile && env.resolver.exists(projectFile)) {
        const projectId = file_1.readProjectIdFromProjectFile(env.resolver, projectFile);
        if (projectId) {
            return projectId;
        }
    }
}
function handleKeyEvent(name, currentIndex, selectedIndices, projects, env, callback) {
    return __awaiter(this, void 0, void 0, function* () {
        switch (name) {
            case 'DOWN': {
                currentIndex = (currentIndex + 1) % projects.length;
                rerender(projects, currentIndex, selectedIndices);
                break;
            }
            case 'UP': {
                currentIndex = (currentIndex + projects.length - 1) % projects.length;
                rerender(projects, currentIndex, selectedIndices);
                break;
            }
            case ' ': {
                const index = selectedIndices.indexOf(currentIndex);
                if (index >= 0) {
                    selectedIndices.splice(index, 1);
                }
                else {
                    selectedIndices.push(currentIndex);
                }
                rerender(projects, currentIndex, selectedIndices);
                break;
            }
            case 'ENTER': {
                yield handleSelect(selectedIndices, projects, env);
                terminal.grabInput(false);
                callback();
                break;
            }
            case 'CTRL_C': {
                clear(projects);
                terminal.hideCursor(false);
                terminal.grabInput(false);
                process.exit();
            }
            default: {
                break;
            }
        }
        return currentIndex;
    });
}
function handleSelect(selectedIndices, projects, env) {
    return __awaiter(this, void 0, void 0, function* () {
        terminal(`\n\n${constants_1.deletingProjectWarningMessage}`);
        terminal.grabInput(true);
        yield new Promise(resolve => {
            terminal.on('key', function (name) {
                if (name !== 'y') {
                    process.exit(0);
                }
                terminal.grabInput(false);
                resolve();
            });
        });
        const projectIdsToDelete = selectedIndices.reduce((prev, current) => {
            prev.push(projects[current].projectId);
            return prev;
        }, []);
        terminal.restoreCursor();
        terminal.eraseDisplayBelow();
        terminal.hideCursor(false);
        env.out.startSpinner(constants_1.deletingProjectsMessage(projectIdsToDelete));
        try {
            yield api_1.deleteProject(projectIdsToDelete, env.resolver);
            env.out.stopSpinner();
            env.out.write(constants_1.deletedProjectMessage(projectIdsToDelete));
        }
        catch (e) {
            env.out.stopSpinner();
            if (e.errors) {
                const errors = api_1.parseErrors(e);
                const output = api_1.generateErrorOutput(errors);
                env.out.writeError(`${output}`);
            }
            else {
                throw e;
            }
        }
    });
}
function rerender(projects, currentIndex, selectedIndices) {
    clear(projects);
    render(projects, currentIndex, selectedIndices);
}
function clear(projects) {
    const lineCount = _.flatten(projects).length - 1;
    terminal.up(lineCount);
    terminal.left(10000);
    terminal.eraseDisplayBelow();
}
function render(projects, currentIndex, selectedIndices) {
    const lines = _.chain(projects)
        .map(project => `${project.name} (${project.projectId})`)
        .map((l, lineIndex) => (selectedIndices.includes(lineIndex)) ? `${chalk.red(figures.circleFilled)}  ${chalk.red(l)}` : `${figures.circle}  ${l}`)
        .map((l, lineIndex) => (lineIndex === currentIndex) ? `${figures.pointer} ${l}` : `  ${l}`)
        .join('\n');
    terminal(lines, currentIndex);
}
//# sourceMappingURL=delete.js.map