"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const constants_1 = require("../utils/constants");
const chalk = require("chalk");
const figures = require("figures");
const _ = require("lodash");
const init_1 = require("./init");
const file_1 = require("../utils/file");
const { terminal } = require('terminal-kit');
const debug = require('debug')('graphcool');
const BLANK_PROJECT = 0;
exports.default = (props, env) => __awaiter(this, void 0, void 0, function* () {
    const { out, resolver } = env;
    const schemaFiles = resolver.schemaFiles('.');
    const projectFiles = resolver.projectFiles('.');
    let options = [
        [`${chalk.bold('New blank project')}`, `Creates a new Graphcool project from scratch.`, ''],
        ...schemaFiles.map(f => [`${chalk.bold('From local schema ./' + f)}`, 'Creates a new Graphcool project based on the local schema.', '']),
    ];
    // only recommend copy when no region has been specified, because copy across regions is not yet possible
    if (!props.region) {
        options = [
            ...options,
            ...projectFiles.map(f => [`${chalk.bold('Copy existing project ./' + f)}`, 'Creates a clone of an existing Graphcool project.', '']),
        ];
    }
    // no need for interactivity when there are no options
    // NOTE: this should probably be refactored to an outer layer
    if (schemaFiles.length === 0 && projectFiles.length === 0) {
        yield props.checkAuth(env, 'init');
        const schemaUrl = constants_1.sampleSchemaURL;
        const initProps = getPropsForInit(props);
        yield init_1.default(Object.assign({}, initProps, { schemaUrl }), env);
        return;
    }
    terminal.saveCursor();
    out.write(constants_1.howDoYouWantToGetStarted());
    terminal.grabInput();
    terminal.hideCursor();
    let currentIndex = BLANK_PROJECT; // 0
    render(options, currentIndex);
    yield new Promise(resolve => {
        terminal.on('key', (name) => __awaiter(this, void 0, void 0, function* () {
            currentIndex = yield handleKeyEvent(name, currentIndex, options, props, env, resolve);
        }));
    });
});
function handleKeyEvent(name, currentIndex, options, props, env, callback) {
    return __awaiter(this, void 0, void 0, function* () {
        switch (name) {
            case 'DOWN': {
                currentIndex = (currentIndex + 1) % options.length;
                rerender(options, currentIndex);
                break;
            }
            case 'UP': {
                currentIndex = (currentIndex + options.length - 1) % options.length;
                rerender(options, currentIndex);
                break;
            }
            case 'ENTER': {
                yield handleSelect(currentIndex, props, env);
                callback();
                break;
            }
            case 'CTRL_C': {
                terminal.restoreCursor();
                terminal.eraseDisplayBelow();
                terminal.hideCursor(false);
                env.out.write('\n');
                process.exit();
            }
            default: {
                break;
            }
        }
        return currentIndex;
    });
}
function handleSelect(selectedIndex, props, env) {
    return __awaiter(this, void 0, void 0, function* () {
        terminal.restoreCursor();
        terminal.eraseDisplayBelow();
        terminal.hideCursor(false);
        env.out.write('\n');
        if (selectedIndex === BLANK_PROJECT) {
            terminal.grabInput(false);
            yield props.checkAuth(env, 'init');
        }
        switch (selectedIndex) {
            case BLANK_PROJECT: {
                const schemaUrl = constants_1.sampleSchemaURL;
                const initProps = getPropsForInit(props);
                yield init_1.default(Object.assign({}, initProps, { schemaUrl }), env);
                break;
            }
            default: {
                terminal.grabInput(false);
                const schemaFiles = env.resolver.schemaFiles('.');
                const projectFiles = env.resolver.projectFiles('.');
                const previousOptions = 1;
                if (selectedIndex >= previousOptions && selectedIndex < previousOptions + schemaFiles.length) {
                    const schemaFileIndex = selectedIndex - previousOptions;
                    const schemaUrl = schemaFiles[schemaFileIndex];
                    const initProps = getPropsForInit(props);
                    yield init_1.default(Object.assign({}, initProps, { schemaUrl }), env);
                }
                else if (selectedIndex >= previousOptions + schemaFiles.length && selectedIndex < previousOptions + schemaFiles.length + projectFiles.length) {
                    const projectFileIndex = selectedIndex - schemaFiles.length - previousOptions;
                    const projectFile = projectFiles[projectFileIndex];
                    const copyProjectId = file_1.readProjectIdFromProjectFile(env.resolver, projectFile);
                    const initProps = getPropsForInit(props);
                    const initProps2 = Object.assign({}, initProps, { copyProjectId, projectFile });
                    yield init_1.default(initProps2, env);
                }
                break;
            }
        }
    });
}
function getPropsForInit(props) {
    return {
        name: props.name,
        alias: props.alias,
        region: props.region,
        outputPath: props.outputPath
    };
}
function rerender(options, currentIndex) {
    clear(options);
    render(options, currentIndex);
}
function clear(options) {
    const lineCount = _.flatten(options).length - 1;
    terminal.up(lineCount);
    terminal.left(10000);
    terminal.eraseDisplayBelow();
}
function render(options, currentIndex) {
    const lines = _.chain(options)
        .map((ls, optionIndex) => ls.map((l, lineIndex) => (lineIndex === 0 && optionIndex === currentIndex) ? `  ${chalk.blue(figures.pointer)} ${l}` : `    ${l}`))
        .flatten()
        .join('\n');
    terminal(lines, currentIndex);
}
//# sourceMappingURL=interactiveInit.js.map