"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const api_1 = require("../api/api");
const figures = require("figures");
const chalk = require("chalk");
const constants_1 = require("../utils/constants");
const file_1 = require("../utils/file");
const utils_1 = require("../utils/utils");
const debug = require('debug')('graphcool');
exports.default = (props, env) => __awaiter(this, void 0, void 0, function* () {
    const { resolver, out } = env;
    const projectFilePath = getProjectFilePath(props, resolver);
    const projectInfo = file_1.readProjectInfoFromProjectFile(resolver, projectFilePath);
    if (!projectInfo) {
        throw new Error(constants_1.invalidProjectFileMessage);
    }
    const { projectId, schema, version } = projectInfo;
    const force = props.force;
    out.startSpinner(constants_1.pushingNewSchemaMessage);
    try {
        // first compare local and remote versions and fail if remote is ahead
        const remoteProjectInfo = yield api_1.pullProjectInfo(projectInfo.projectId, resolver);
        if (parseInt(projectInfo.version) > parseInt(remoteProjectInfo.version)) {
            out.stopSpinner();
            throw new Error(constants_1.remoteSchemaAheadMessage(remoteProjectInfo.version, projectInfo.version));
        }
        const schemaWithFrontmatter = `# project: ${projectId}\n# version: ${version}\n\n${schema}`;
        const migrationResult = yield api_1.pushNewSchema(schemaWithFrontmatter, force, resolver);
        out.stopSpinner();
        // no action required
        if (migrationResult.messages.length === 0 && migrationResult.errors.length === 0) {
            out.write(constants_1.noActionRequiredMessage);
            return;
        }
        else if (migrationResult.messages.length > 0 && migrationResult.errors.length === 0) {
            const migrationMessage = constants_1.migrationPerformedMessage;
            out.write(`${migrationMessage}`);
            printMigrationMessages(migrationResult.messages, out);
            out.write(`\n`);
            // update project file if necessary
            const projectInfo = {
                projectId,
                schema: migrationResult.newSchema,
                version: migrationResult.newVersion
            };
            file_1.writeProjectFile(projectInfo, resolver, projectFilePath);
            out.write(constants_1.projectFileWasUpdatedMessage(projectFilePath));
        }
        else if (migrationResult.messages.length === 0 && migrationResult.errors.length > 0) {
            out.write(`${constants_1.migrationErrorMessage}`);
            printMigrationErrors(migrationResult.errors, out);
            out.write(`\n`);
        }
        else if (migrationResult.errors[0].description.indexOf(`destructive changes`) >= 0) {
            out.write(constants_1.potentialDataLossMessage);
        }
    }
    catch (e) {
        out.stopSpinner();
        if (e.errors) {
            const errors = api_1.parseErrors(e);
            const output = api_1.generateErrorOutput(errors);
            out.writeError(`${output}`);
        }
        else {
            throw e;
        }
    }
});
function getProjectFilePath(props, resolver) {
    // check if provided file is valid (ends with correct suffix)
    if (props.projectFile && file_1.isValidProjectFilePath(props.projectFile)) {
        return props.projectFile;
    }
    else if (props.projectFile && !file_1.isValidProjectFilePath(props.projectFile)) {
        throw new Error(constants_1.invalidProjectFilePathMessage(props.projectFile));
    }
    // no project file provided, search for one in current dir
    const projectFiles = resolver.projectFiles('.');
    if (projectFiles.length === 0) {
        throw new Error(constants_1.noProjectFileForPushMessage);
    }
    else if (projectFiles.length > 1) {
        throw new Error(constants_1.multipleProjectFilesMessage(projectFiles));
    }
    return projectFiles[0];
}
function printMigrationMessages(migrationMessages, out) {
    migrationMessages.forEach((migrationMessage, index) => {
        const actionType = getMigrationActionType(migrationMessage.description);
        const symbol = getSymbolForMigrationActionType(actionType);
        const description = utils_1.makePartsEnclodesByCharacterBold(migrationMessage.description, `\``);
        const outputMessage = `${index > 0 ? `  |` : ``}\n  | (${symbol})  ${description}\n`;
        out.write(outputMessage);
        migrationMessage.subDescriptions.forEach(subMessage => {
            const actionType = getMigrationActionType(subMessage.description);
            const symbol = getSymbolForMigrationActionType(actionType);
            const outputMessage = utils_1.makePartsEnclodesByCharacterBold(subMessage.description, `\``);
            out.write(`  ├── (${symbol})  ${outputMessage}\n`);
        });
    });
}
function getMigrationActionType(message) {
    if (message.indexOf('create') >= 0) {
        return 'create';
    }
    else if (message.indexOf('update') >= 0) {
        return 'update';
    }
    else if (message.indexOf('delete') >= 0 || message.indexOf('remove') >= 0) {
        return 'delete';
    }
    return 'unknown';
}
function getSymbolForMigrationActionType(type) {
    switch (type) {
        case 'create': return '+';
        case 'delete': return '-';
        case 'update': return '*';
        case 'unknown': return '?';
    }
}
function printMigrationErrors(errors, out) {
    const indentation = spaces(2);
    errors.forEach(error => {
        const outputMessage = utils_1.makePartsEnclodesByCharacterBold(error.description, `\``);
        out.write(`${indentation}${chalk.red(figures.cross)} ${outputMessage}\n`);
    });
}
const spaces = (n) => Array(n + 1).join(' ');
//# sourceMappingURL=push.js.map