"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const file_1 = require("../utils/file");
const constants_1 = require("../utils/constants");
const api_1 = require("../api/api");
const debug = require('debug')('graphcool');
const figures = require("figures");
const chalk = require("chalk");
const utils_1 = require("../utils/utils");
exports.default = (props, env) => __awaiter(this, void 0, void 0, function* () {
    const { resolver, out } = env;
    try {
        const projectFile = getProjectFilePath(props, resolver);
        const projectId = file_1.readProjectIdFromProjectFile(resolver, projectFile);
        if (!projectId) {
            throw new Error(constants_1.noProjectIdMessage);
        }
        const localVersion = file_1.readVersionFromProjectFile(resolver, projectFile);
        if (!localVersion || !parseInt(localVersion)) {
            throw new Error(constants_1.canNotReadVersionFromProjectFile(projectFile));
        }
        const localSchema = file_1.readDataModelFromProjectFile(resolver, projectFile);
        const schemaWithFrontmatter = `# project: ${projectId}\n# version: ${localVersion}\n\n${localSchema}`;
        const migrationResult = yield api_1.statusMessage(schemaWithFrontmatter, resolver);
        out.write(constants_1.statusHeaderMessage(projectId, migrationResult.newVersion));
        if (parseInt(localVersion) < parseInt(migrationResult.newVersion)) {
            out.write(constants_1.localSchemaBehindRemoteMessage(migrationResult.newVersion, localVersion));
            return;
        }
        else if (parseInt(localVersion) > parseInt(migrationResult.newVersion)) {
            throw new Error(constants_1.remoteSchemaBehindLocalMessage(migrationResult.newVersion, localVersion));
        }
        // no action required
        if (migrationResult.messages.length === 0 && migrationResult.errors.length === 0) {
            out.write(constants_1.everythingUpToDateMessage);
            return;
        }
        else if (migrationResult.messages.length > 0 && migrationResult.errors.length === 0) {
            const migrationMessage = constants_1.potentialChangesMessage;
            out.write(`${migrationMessage}`);
            printMigrationMessages(migrationResult.messages, out);
            out.write(constants_1.usePushToUpdateMessage);
        }
        else if (migrationResult.messages.length === 0 && migrationResult.errors.length > 0) {
            const migrationMessage = constants_1.potentialChangesMessage;
            out.write(`${constants_1.issuesInSchemaMessage}`);
            printMigrationErrors(migrationResult.errors, out);
        }
        else if (migrationResult.errors[0].description.indexOf(`destructive changes`) >= 0) {
            const migrationMessage = constants_1.potentialChangesMessage;
            out.write(`${migrationMessage}`);
            printMigrationMessages(migrationResult.messages, out);
            out.write(constants_1.destructiveChangesInStatusMessage);
        }
    }
    catch (e) {
        out.stopSpinner();
        if (e.errors) {
            const errors = api_1.parseErrors(e);
            const output = api_1.generateErrorOutput(errors);
            out.writeError(`${output}`);
        }
        else {
            throw e;
        }
    }
});
function getProjectFilePath(props, resolver) {
    // check if provided file is valid (ends with correct suffix)
    if (props.projectFile && file_1.isValidProjectFilePath(props.projectFile)) {
        return props.projectFile;
    }
    else if (props.projectFile && !file_1.isValidProjectFilePath(props.projectFile)) {
        throw new Error(constants_1.invalidProjectFilePathMessage(props.projectFile));
    }
    // no project file provided, search for one in current dir
    const projectFiles = resolver.projectFiles('.');
    if (projectFiles.length === 0) {
        throw new Error(constants_1.noProjectFileMessage);
    }
    else if (projectFiles.length > 1) {
        throw new Error(constants_1.multipleProjectFilesForStatusMessage(projectFiles));
    }
    return projectFiles[0];
}
function printMigrationMessages(migrationMessages, out) {
    migrationMessages.forEach((migrationMessage, index) => {
        const actionType = getMigrationActionType(migrationMessage.description);
        const symbol = getSymbolForMigrationActionType(actionType);
        const description = utils_1.makePartsEnclodesByCharacterBold(migrationMessage.description, `\``);
        const outputMessage = `${index > 0 ? `  |` : ``}\n  | (${symbol})  ${description}\n`;
        out.write(outputMessage);
        migrationMessage.subDescriptions.forEach(subMessage => {
            const actionType = getMigrationActionType(subMessage.description);
            const symbol = getSymbolForMigrationActionType(actionType);
            const outputMessage = utils_1.makePartsEnclodesByCharacterBold(subMessage.description, `\``);
            out.write(`  ├── (${symbol})  ${outputMessage}\n`);
        });
    });
}
function getMigrationActionType(message) {
    if (message.indexOf('create') >= 0) {
        return 'create';
    }
    else if (message.indexOf('update') >= 0) {
        return 'update';
    }
    else if (message.indexOf('delete') >= 0 || message.indexOf('remove') >= 0) {
        return 'delete';
    }
    return 'unknown';
}
function getSymbolForMigrationActionType(type) {
    switch (type) {
        case 'create': return '+';
        case 'delete': return '-';
        case 'update': return '*';
        case 'unknown': return '?';
    }
}
function printMigrationErrors(errors, out) {
    const indentation = spaces(2);
    errors.forEach(error => {
        const outputMessage = utils_1.makePartsEnclodesByCharacterBold(error.description, `\``);
        out.write(`${indentation}${chalk.red(figures.cross)} ${outputMessage}\n`);
    });
}
const spaces = (n) => Array(n + 1).join(' ');
//# sourceMappingURL=status.js.map