#!/usr/bin/env node
"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const minimist = require("minimist");
const push_1 = require("./commands/push");
const console_1 = require("./commands/console");
const playground_1 = require("./commands/playground");
const projects_1 = require("./commands/projects");
const pull_1 = require("./commands/pull");
const auth_1 = require("./commands/auth");
const init_1 = require("./commands/init");
const interactiveInit_1 = require("./commands/interactiveInit");
const export_1 = require("./commands/export");
const endpoints_1 = require("./commands/endpoints");
const status_1 = require("./commands/status");
const quickstart_1 = require("./commands/quickstart");
const delete_1 = require("./commands/delete");
const FileSystemResolver_1 = require("./system/FileSystemResolver");
const StdOut_1 = require("./system/StdOut");
const GraphcoolAuthServer_1 = require("./api/GraphcoolAuthServer");
const constants_1 = require("./utils/constants");
const usage_1 = require("./utils/usage");
const arguments_1 = require("./utils/arguments");
const Raven = require('raven');
const debug = require('debug')('graphcool');
const { version } = require('../../package.json');
const config_1 = require("./utils/config");
function main() {
    return __awaiter(this, void 0, void 0, function* () {
        // initialize sentry
        Raven.config(constants_1.sentryDSN).install();
        const env = defaultEnvironment();
        const argv = minimist(process.argv.slice(2));
        const command = argv._[0];
        if (command) {
            checkHelp(command, argv);
            checkOptions(command, argv);
        }
        const displayQuickstart = shouldDisplayQuickstart();
        switch (command) {
            case undefined: {
                if (argv['version'] || argv['v']) {
                    process.stdout.write(version);
                    break;
                }
                process.stdout.write(usage_1.usageRoot(displayQuickstart));
                process.exit(0);
            }
            // TODO remove legacy support
            case 'create': {
                yield checkAuth(env, 'auth');
                console.log('`graphcool create` is deprecated and will be removed in a future version. Use `graphcool init` instead.');
                const name = argv['name'] || argv['n'];
                const alias = argv['alias'] || argv['a'];
                const region = argv['region'] || argv['r'];
                const schemaUrl = argv._[1];
                const remoteSchemaUrl = schemaUrl;
                const localSchemaFile = schemaUrl;
                const props = { name, alias, remoteSchemaUrl, localSchemaFile, region };
                yield init_1.default(props, env);
                break;
            }
            case 'init': {
                const name = argv['name'] || argv['n'];
                const alias = argv['alias'] || argv['a'];
                const region = argv['region'] || argv['r'];
                // if there are exactly 2 argv's, use the 2nd argument as an alternative for the schemaUrl
                const schemaUrl = argv['schema'] || argv['s'] || argv.length === 2 && argv._[1];
                const copyProjectId = argv['copy'] || argv['c'];
                const copyOptions = argv['copy-options'];
                const outputPath = argv['output'] || argv['o'];
                if (!schemaUrl && !copyProjectId) {
                    const props = { name, alias, outputPath, checkAuth, region };
                    yield interactiveInit_1.default(props, env);
                }
                else {
                    yield checkAuth(env, 'init');
                    const props = { name, alias, schemaUrl, copyProjectId, copyOptions, region, outputPath };
                    yield init_1.default(props, env);
                }
                break;
            }
            case 'push': {
                yield checkAuth(env, 'auth');
                checkOptions('push', argv);
                const force = !!(argv['force'] || argv['f']);
                const projectFile = argv._[1];
                yield push_1.default({ force, projectFile }, env);
                break;
            }
            case 'delete': {
                yield checkAuth(env, 'auth');
                const sourceProjectId = argv['project'] || argv['p'];
                yield delete_1.default({ sourceProjectId }, env);
                break;
            }
            case 'pull': {
                yield checkAuth(env, 'auth');
                const sourceProjectId = argv['project'] || argv['p'];
                const projectFile = argv._[1];
                const outputPath = argv['output'] || argv['o'];
                const force = !!(argv['force'] || argv['f']);
                const props = { sourceProjectId, projectFile, outputPath, force };
                yield pull_1.default(props, env);
                break;
            }
            case 'export': {
                yield checkAuth(env, 'auth');
                const projectFile = argv._[1];
                yield export_1.default({ projectFile }, env);
                break;
            }
            case 'status': {
                yield checkAuth(env, 'auth');
                const projectFile = argv._[1];
                yield status_1.default({ projectFile }, env);
                break;
            }
            case 'endpoints': {
                yield checkAuth(env, 'auth');
                const projectFile = argv._[1];
                yield endpoints_1.default({ projectFile }, env);
                break;
            }
            case 'console': {
                yield checkAuth(env, 'auth');
                const projectFile = argv._[1];
                yield console_1.default({ projectFile }, env);
                break;
            }
            case 'playground': {
                yield checkAuth(env, 'auth');
                const projectFile = argv._[1];
                yield playground_1.default({ projectFile }, env);
                break;
            }
            case 'projects': {
                yield checkAuth(env, 'auth');
                yield projects_1.default({}, env);
                break;
            }
            case 'auth': {
                const token = argv['token'] || argv['t'];
                yield auth_1.default({ token }, env, new GraphcoolAuthServer_1.GraphcoolAuthServer('auth'));
                break;
            }
            case 'quickstart': {
                const props = { checkAuth };
                yield quickstart_1.default(props, env);
                break;
            }
            case 'help': {
                process.stdout.write(usage_1.usageRoot(displayQuickstart));
                process.exit(0);
                break;
            }
            case 'version': {
                process.stdout.write(version);
                break;
            }
            default: {
                if (argv['version'] || argv['v']) {
                    process.stdout.write(version);
                    break;
                }
                process.stdout.write(`Unknown command: ${command}\n\n${usage_1.usageRoot(displayQuickstart)}`);
                break;
            }
        }
        process.stdout.write('\n');
    });
}
function checkAuth(env, authTrigger) {
    return __awaiter(this, void 0, void 0, function* () {
        if (env.config.get('token')) {
            return true;
        }
        yield auth_1.default({}, env, new GraphcoolAuthServer_1.GraphcoolAuthServer(authTrigger));
        return false;
    });
}
function shouldDisplayQuickstart() {
    const resolver = new FileSystemResolver_1.default();
    if (resolver.exists(constants_1.graphcoolConfigFilePath)) {
        const content = JSON.parse(resolver.read(constants_1.graphcoolConfigFilePath));
        if (content.token && content.token.length > 0) {
            return false;
        }
    }
    return true;
}
function checkHelp(command, argv) {
    const usage = arguments_1.usageForCommand(command);
    if (argv['help'] || argv['h']) {
        process.stdout.write(usage);
        process.exit(0);
    }
}
function checkOptions(command, inputArgs) {
    const allowedOptions = arguments_1.optionsForCommand(command);
    const input = Object.keys(inputArgs);
    const unknownOptions = input.filter(x => allowedOptions.indexOf(x) < 0).filter(x => x !== '_');
    if (unknownOptions.length > 0) {
        const errorMessage = constants_1.unknownOptionsWarning(command, unknownOptions);
        process.stderr.write(errorMessage);
        process.exit(0);
    }
}
function defaultEnvironment() {
    const resolver = new FileSystemResolver_1.default();
    var config = new config_1.Config(resolver);
    config.load();
    return {
        resolver: resolver,
        out: new StdOut_1.default(),
        config: config
    };
}
process.on('unhandledRejection', e => new StdOut_1.default().onError(e));
main();
//# sourceMappingURL=index.js.map