"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const ava_1 = require("ava");
const test_environment_1 = require("./helpers/test_environment");
const init_1 = require("../src/commands/init");
const constants_1 = require("../src/utils/constants");
const mock_data_1 = require("./fixtures/mock_data");
const file_1 = require("../src/utils/file");
const ping_1 = require("../src/utils/ping");
const schemas_1 = require("./fixtures/schemas");
require("isomorphic-fetch");
const test_out_1 = require("./helpers/test_out");
const fetchMock = require('fetch-mock');
const debug = require('debug')('graphcool');
/*
 Tests:
 - Succeeding project creation with local schema file
 - Succeeding project creation with remote schema file
 - Succeeding project creation with local file and different output path
 - Succeeding project creation with local schema file and custom region
 - Succeeding project creation because with invalid output path, falls back to default
 - Succeeding project creation with alias
 - Failing project creation with an invalid project name
 */
ava_1.default.beforeEach(() => {
    fetchMock.mock(`glob:${ping_1.getDynamoUrl('*')}*`, 404);
});
ava_1.default.afterEach(() => {
    new test_out_1.default().write('\n');
    fetchMock.restore();
});
ava_1.default('Succeeding project creation with local schema file', (t) => __awaiter(this, void 0, void 0, function* () {
    const name = 'MyProject';
    const schemaUrl = './myproject.graphql';
    // configure HTTP mocks
    fetchMock.post(constants_1.systemAPIEndpoint, JSON.parse(mock_data_1.mockedCreateProjectResponse));
    // create dummy project data
    const props = { name, schemaUrl };
    // prepare environment
    const env = test_environment_1.testEnvironment({});
    env.resolver.write(schemaUrl, schemas_1.simpleTwitterSchema);
    env.resolver.write(constants_1.graphcoolConfigFilePath, '{"token": "abcdefgh"}');
    env.out.prefix(t._test.title, `$ graphcool init -s ${schemaUrl} -n ${name}`);
    yield t.notThrows(init_1.default(props, env));
    const expectedProjectFileContent = mock_data_1.mockProjectFile1;
    t.is(env.resolver.read(constants_1.graphcoolProjectFileName), expectedProjectFileContent);
    t.is(file_1.readProjectIdFromProjectFile(env.resolver, constants_1.graphcoolProjectFileName), 'abcdefghijklmn');
    t.is(file_1.readVersionFromProjectFile(env.resolver, constants_1.graphcoolProjectFileName), '1');
}));
ava_1.default('Succeeding project creation with local schema file and lowercase region', (t) => __awaiter(this, void 0, void 0, function* () {
    const name = 'MyProject';
    const schemaUrl = './myproject.graphql';
    const region = 'eu_west_1';
    // configure HTTP mocks
    fetchMock.post(constants_1.systemAPIEndpoint, JSON.parse(mock_data_1.mockedCreateProjectResponse));
    // create dummy project data
    const props = { name, schemaUrl, region };
    // prepare environment
    const env = test_environment_1.testEnvironment({});
    env.resolver.write(schemaUrl, schemas_1.simpleTwitterSchema);
    env.resolver.write(constants_1.graphcoolConfigFilePath, '{"token": "abcdefgh"}');
    env.out.prefix(t._test.title, `$ graphcool init -s ${schemaUrl} -n ${name} -r ${region}`);
    yield t.notThrows(init_1.default(props, env));
    const expectedProjectFileContent = mock_data_1.mockProjectFile1;
    t.is(env.resolver.read(constants_1.graphcoolProjectFileName), expectedProjectFileContent);
    t.is(file_1.readProjectIdFromProjectFile(env.resolver, constants_1.graphcoolProjectFileName), 'abcdefghijklmn');
    t.is(file_1.readVersionFromProjectFile(env.resolver, constants_1.graphcoolProjectFileName), '1');
}));
ava_1.default('Succeeding project creation with remote schema file', (t) => __awaiter(this, void 0, void 0, function* () {
    const name = 'MyProject';
    const schemaUrl = 'https://graphqlbin/project.graphql';
    // configure HTTP mocks
    fetchMock.post(constants_1.systemAPIEndpoint, JSON.parse(mock_data_1.mockedCreateProjectResponse));
    const schema = schemas_1.simpleTwitterSchema;
    fetchMock.get(schemaUrl, schema);
    // create dummy project data
    const props = { name, schemaUrl };
    // prepare environment
    const env = test_environment_1.testEnvironment({});
    env.resolver.write(constants_1.graphcoolConfigFilePath, '{"token": "abcdefgh"}');
    env.out.prefix(t._test.title, `$ graphcool init -s ${schemaUrl} -n ${name}`);
    yield t.notThrows(init_1.default(props, env));
    const expectedProjectFileContent = mock_data_1.mockProjectFile1;
    t.is(env.resolver.read(constants_1.graphcoolProjectFileName), expectedProjectFileContent);
    t.is(file_1.readProjectIdFromProjectFile(env.resolver, constants_1.graphcoolProjectFileName), 'abcdefghijklmn');
    t.is(file_1.readVersionFromProjectFile(env.resolver, constants_1.graphcoolProjectFileName), '1');
}));
ava_1.default('Succeeding project creation with local file and different output path', (t) => __awaiter(this, void 0, void 0, function* () {
    const name = 'MyProject';
    const schemaUrl = './myproject.graphql';
    const outputPath = '/Desktop/example.graphcool';
    // configure HTTP mocks
    fetchMock.post(constants_1.systemAPIEndpoint, JSON.parse(mock_data_1.mockedCreateProjectResponse));
    // create dummy project data
    const props = { name, schemaUrl, outputPath };
    // prepare environment
    const env = test_environment_1.testEnvironment({});
    env.resolver.write(schemaUrl, schemas_1.simpleTwitterSchema);
    env.resolver.write(constants_1.graphcoolConfigFilePath, '{"token": "abcdefgh"}');
    env.out.prefix(t._test.title, `$ graphcool init -s ${schemaUrl} -n ${name} -o ${outputPath}`);
    yield t.notThrows(init_1.default(props, env));
    const expectedProjectFileContent = mock_data_1.mockProjectFile1;
    t.is(env.resolver.read(outputPath), expectedProjectFileContent);
    t.is(file_1.readProjectIdFromProjectFile(env.resolver, outputPath), 'abcdefghijklmn');
    t.is(file_1.readVersionFromProjectFile(env.resolver, outputPath), '1');
}));
ava_1.default('Succeeding project creation with invalid output path, falls back to default', (t) => __awaiter(this, void 0, void 0, function* () {
    const name = 'MyProject';
    const schemaUrl = './myproject.graphql';
    const outputPath = '/Desktop/example.graphql';
    // configure HTTP mocks
    fetchMock.post(constants_1.systemAPIEndpoint, JSON.parse(mock_data_1.mockedCreateProjectResponse));
    // create dummy project data
    const props = { name, schemaUrl, outputPath };
    // prepare environment
    const env = test_environment_1.testEnvironment({});
    env.resolver.write(schemaUrl, schemas_1.simpleTwitterSchema);
    env.resolver.write(constants_1.graphcoolConfigFilePath, '{"token": "abcdefgh"}');
    env.out.prefix(t._test.title, `$ graphcool init -s ${schemaUrl} -n ${name} -o ${outputPath}`);
    yield t.notThrows(init_1.default(props, env));
    const expectedProjectFileContent = mock_data_1.mockProjectFile1;
    t.is(env.resolver.read(constants_1.graphcoolProjectFileName), expectedProjectFileContent);
}));
ava_1.default('Succeeding project creation with alias', (t) => __awaiter(this, void 0, void 0, function* () {
    const name = 'MyProject';
    const schemaUrl = './myproject.graphql';
    const alias = 'example';
    // configure HTTP mocks
    fetchMock.post(constants_1.systemAPIEndpoint, JSON.parse(mock_data_1.mockedCreateProjectResponseWithAlias));
    // create dummy project data
    const props = { name, alias, schemaUrl };
    // prepare environment
    const env = test_environment_1.testEnvironment({});
    env.resolver.write(schemaUrl, schemas_1.simpleTwitterSchema);
    env.resolver.write(constants_1.graphcoolConfigFilePath, '{"token": "abcdefgh"}');
    env.out.prefix(t._test.title, `$ graphcool init -s ${schemaUrl} -n ${name} -a ${alias}`);
    yield t.notThrows(init_1.default(props, env));
    const expectedProjectFileContent = mock_data_1.mockProjectFileWithAlias1;
    t.is(env.resolver.read(constants_1.graphcoolProjectFileName), expectedProjectFileContent);
    t.is(file_1.readProjectIdFromProjectFile(env.resolver, constants_1.graphcoolProjectFileName), alias);
    t.is(file_1.readVersionFromProjectFile(env.resolver, constants_1.graphcoolProjectFileName), '1');
}));
ava_1.default('Failing project creation with an invalid project name', (t) => __awaiter(this, void 0, void 0, function* () {
    const name = 'myProject';
    // create dummy project data
    const props = { name };
    // prepare environment
    const env = test_environment_1.testEnvironment({});
    env.resolver.write(constants_1.graphcoolConfigFilePath, '{"token": "abcdefgh"}');
    env.out.prefix(t._test.title, `$ graphcool init -n ${name}`);
    const error = yield t.throws(init_1.default(props, env));
    t.is(error.message.trim(), `\'myProject\' is not a valid project name. It must begin with an uppercase letter.`);
}));
//# sourceMappingURL=init.test.js.map